﻿local L = AceLibrary("AceLocale-2.2"):new("Automaton_Summon")
local abacus = AceLibrary("Abacus-2.0")

Automaton_Summon = Automaton:NewModule("Summon", "AceEvent-2.0", "AceConsole-2.0", "AceDebug-2.0")
local module = Automaton_Summon
local seconds

L:RegisterTranslations("enUS", function() return {
	["Summon"] = true,
	["Options for accepting summons."] = true,
	["Enabled"] = true,
	["Accept warlock or meeting stone summons automatically."] = true,
	["Delay"] = true,
	["With this option enabled, automatic summons are delayed."] = true,
	["Delay Time"] = true,
	["Set the time in seconds to delay automatic summon acceptance."] = true,
	["Combat Delay"] = true,
	["If summoned while in combat, accept the summon after combat ends."] = true,
	["Combat Delay Time"] = true,
	["Set the time in seconds to delay automatic summon acceptance after combat ends."] = true,
	["Be Quiet!"] = true,
	["Suppress chat frame output from the Summon module."] = true,
	["Accepting summon in %s..."] = true,
	["Accepting summon..."] = true,
	["Cancelling summon..."] = true,
	["In combat! Accepting summon after combat ends..."] = true,
	["Left combat. Accepting summon in %s..."] = true,
	["Summon expired!"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Summon"] = "자동 소환",
	["Options for accepting summons."] = "소환에 관한 옵션을 설정합니다",
	["Enabled"] = "활성화",
	["Accept warlock or meeting stone summons automatically."] = "자동으로 흑마법사 또는 만남의 돌의 소환을 수락합니다.",
	["Delay"] = "지연",
	["With this option enabled, automatic summons are delayed."] = "이 옵션이 활성화시, 자동 소환을 %s 동안 지연시킵니다.",
	["Delay Time"] = "지연 시간",
	["Set the time in seconds to delay automatic summon acceptance."] = "자동 소환을 수락할 시간을 설정합니다",
	["Combat Delay"] = "전투 지연",
	["If summoned while in combat, accept the summon after combat ends."] = "전투 종료후 소환을 받아드립니다.",
	["Combat Delay Time"] = "전투 지연 시간",
	["Set the time in seconds to delay automatic summon acceptance after combat ends."] = "전투 종료 후 자동 소환을 수락할 시간을 설정합니다.",
	["Be Quiet!"] = "알림 해제!",
	["Suppress chat frame output from the Summon module."] = "소환 모듈의 대화창 출력을 사용하지 않습니다.",
	["Accepting summon in %s..."] = "%s 소환 수락...",
	["Accepting summon..."] = "소환 수락...",
	["Cancelling summon..."] = "소환 취소...",
	["In combat! Accepting summon after combat ends..."] = "전투 중! 전투 종료 후 소환을 수락합니다...",
	["Left combat. Accepting summon in %s..."] = "전투 종료. %s님의 소환을 수락합니다...",
	["Summon expired!"] = "소환 종료!",
} end)

L:RegisterTranslations("esES", function() return {
	["Summon"] = "Invocaci\195\179n",
	["Options for accepting summons."] = "Opciones para aceptar invocaciones",
	["Enabled"] = "Activado",
	["Accept warlock or meeting stone summons automatically."] = "Acepta las invocaciones de brujos o piedras de encuentro de forma autom\195\161tica",
	["Delay"] = "Retraso",
	["With this option enabled, automatic summons are delayed."] = "Las invocaciones son retrasadas.",
	["Accepting summon in %s..."] = "Aceptando invocaci\195\179n en %s...",
	["Accepting summon..."] = "Aceptando invocaci\195\179n...",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Summon"] = "招喚",
	["Options for accepting summons."] = "自動接受招喚",
	["Enabled"] = "啟用",
	["Accept warlock or meeting stone summons automatically."] = "自動接受招喚",
	["Delay"] = "延遲",
	["With this option enabled, automatic summons are delayed."] = "啟用後延遲%s自動接受招喚",
	["Accepting summon in %s..."] = "%s後自動接受招喚",
	["Accepting summon..."] = "接受招喚...",
} end)

module.description = L["Options for accepting summons."]
module.options = {
	delay = {
		type = "toggle",
		name = L["Delay"],
		desc = L["With this option enabled, automatic summons are delayed."],
		get = function() return module.db.profile.delay end,
		set = function(v) module.db.profile.delay = v end,
	},
	delayTime = {
		type = "range",
		name = L["Delay Time"],
		desc = L["Set the time in seconds to delay automatic summon acceptance."],
		get = function() return module.db.profile.delayTime end,
		set = function(v) module.db.profile.delayTime = v end,
		min = 5,
		max = 105,
		step = 1,
		bigStep = 5,
	},
	combatDelay = {
		type = "toggle",
		name = L["Combat Delay"],
		desc = L["If summoned while in combat, accept the summon after combat ends."],
		get = function() return module.db.profile.combatDelay end,
		set = function(v) module.db.profile.combatDelay = v end,
	},
	combatDelayTime = {
		type = "range",
		name = L["Combat Delay Time"],
		desc = L["Set the time in seconds to delay automatic summon acceptance after combat ends."],
		get = function() return module.db.profile.combatDelayTime end,
		set = function(v) module.db.profile.combatDelayTime = v end,
		min = 5,
		max = 30,
		step = 1,
		bigStep = 5,
	},
	quiet = {
		type = "toggle",
		name = L["Be Quiet!"],
		desc = L["Suppress chat frame output from the Summon module."],
		get = function() return module.db.profile.quiet end,
		set = function(v) module.db.profile.quiet = v end,
	},
}

function module:OnInitialize()
	self.db = Automaton:AcquireDBNamespace("Summon")
	Automaton:RegisterDefaults("Summon", "profile", {
		disabled = true,
		delay = true,
		delayTime = 100,
		combatDelay = true,
		combatDelayTime = 10,
		quiet = false,
	})
	Automaton:SetDisabledAsDefault(self, "Summon")
	self:RegisterOptions(self.options)
end

function module:OnEnable()
	self:RegisterEvent("CONFIRM_SUMMON")

	StaticPopupDialogs["CONFIRM_SUMMON"].OnHide = function()
		if Automaton_Summon:IsEventScheduled("Automaton_SummonAccept") then
			Automaton_Summon:CancelScheduledEvent("Automaton_SummonAccept")
			Automaton_Summon:Print(L["Cancelling summon..."])
		end
		if Automaton_Summon:IsEventScheduled("Automaton_SummonTimeout") then
			Automaton_Summon:CancelScheduledEvent("Automaton_SummonTimeout")
		end
	end
end

function module:OnDisable()
	StaticPopupDialogs["CONFIRM_SUMMON"].OnHide = nil
end

function module:Spam(text)
	if not module.db.profile.quiet then
		self:Print(text)
	end
end

function module:Confirm()
	if UnitAffectingCombat("player") then
		self:Spam(L["In combat! Accepting summon after combat ends..."])
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat")
		self:ScheduleEvent("Automaton_SummonTimeout", self.Timeout, GetSummonConfirmTimeLeft(), self)
		return
	end
	if self:IsEventScheduled("Automaton_SummonTimeout") then
		self:CancelScheduledEvent("Automaton_SummonTimeout")
	end
	ConfirmSummon()
	StaticPopup_Hide("CONFIRM_SUMMON")
end

function module:LeaveCombat()
	seconds = module.db.profile.combatDelayTime
	self:Spam(L["Left combat. Accepting summon in %s..."]:format(abacus:FormatDurationExtended(seconds):lower()))
	self:ScheduleEvent("Automaton_SummonAccept", self.Confirm, seconds, self)
end

function module:Timeout()
	self:Spam(L["Summon expired!"])
	if self:IsEventScheduled("Automaton_SummonAccept") then
		self:CancelScheduledEvent("Automaton_SummonAccept")
	end
	if self:IsEventRegistered("PLAYER_REGEN_ENABLED") then
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	end
end

function module:CONFIRM_SUMMON()
	if module.db.profile.delay and not self:IsEventScheduled("Automaton_SummonAccept") then
		seconds = module.db.profile.delayTime
		self:Spam(L["Accepting summon in %s..."]:format(abacus:FormatDurationExtended(seconds):lower()))
		self:ScheduleEvent("Automaton_SummonAccept", self.Confirm, seconds, self)
	elseif not self:IsEventScheduled("Automaton_SummonAccept") then
		self:Spam(L["Accepting summon..."])
		self:Confirm()
	end
end